@@ -30,6 +30,7 @@ use crate::blaze_err_last;
 use crate::from_cstr;
 use crate::set_last_err;
 use crate::util::slice_from_user_array;
+use crate::util::DynSize as _;
 
 
 /// C ABI compatible version of [`blazesym::inspect::Inspector`].
@@ -179,35 +180,23 @@ pub struct blaze_sym_info {
 }
 
 
-/// Convert [`SymInfo`] objects as returned by
-/// [`Symbolizer::find_addrs`] to a C array.
+/// Convert [`SymInfo`] objects to a C array.
 fn convert_syms_list_to_c(syms_list: Vec<Vec<SymInfo>>) -> *const *const blaze_sym_info {
-    let mut sym_cnt = 0;
-    let mut str_buf_sz = 0;
+    let sym_cnt = syms_list.iter().map(|syms| syms.len() + 1).sum::<usize>();
+    let str_buf_sz = syms_list.c_str_size();
 
-    for syms in &syms_list {
-        sym_cnt += syms.len() + 1;
-        for sym in syms {
-            str_buf_sz += sym.name.len() + 1;
-            if let Some(fname) = sym.module.as_ref() {
-                str_buf_sz += AsRef::<OsStr>::as_ref(fname.deref()).as_bytes().len() + 1;
-            }
-        }
-    }
-
-    let array_sz = (mem::size_of::<*const u64>() * syms_list.len() + mem::size_of::<u64>() - 1)
-        / mem::size_of::<u64>()
+    let array_sz = (mem::size_of::<*const u64>() * syms_list.len()).div_ceil(mem::size_of::<u64>())
         * mem::size_of::<u64>();
     let sym_buf_sz = mem::size_of::<blaze_sym_info>() * sym_cnt;
     let buf_size = mem::size_of::<u64>() + array_sz + sym_buf_sz + str_buf_sz;
-    let raw_buf_with_sz = unsafe { alloc(Layout::from_size_align(buf_size, 8).unwrap()) };
-    if raw_buf_with_sz.is_null() {
+    let buf = unsafe { alloc(Layout::from_size_align(buf_size, 8).unwrap()) };
+    if buf.is_null() {
         return ptr::null()
     }
 
-    unsafe { *(raw_buf_with_sz as *mut u64) = buf_size as u64 };
+    unsafe { *buf.cast::<u64>() = buf_size as u64 };
 
-    let raw_buf = unsafe { raw_buf_with_sz.add(mem::size_of::<u64>()) };
+    let raw_buf = unsafe { buf.add(mem::size_of::<u64>()) };
     let mut syms_ptr = raw_buf as *mut *mut blaze_sym_info;
     let mut sym_ptr = unsafe { raw_buf.add(array_sz) } as *mut blaze_sym_info;
     let mut str_ptr = unsafe { raw_buf.add(array_sz + sym_buf_sz) } as *mut c_char;
@@ -221,6 +210,7 @@ fn convert_syms_list_to_c(syms_list: Vec<Vec<SymInfo>>) -> *const *const blaze_s
             sym_type,
             file_offset,
             module,
+            _non_exhaustive: (),
         } in syms
         {
             let name_ptr = str_ptr.cast();
@@ -347,9 +337,9 @@ pub unsafe extern "C" fn blaze_inspect_syms_free(syms: *const *const blaze_sym_i
         return
     }
 
-    let raw_buf_with_sz = unsafe { (syms as *mut u8).offset(-(mem::size_of::<u64>() as isize)) };
-    let sz = unsafe { *(raw_buf_with_sz as *mut u64) } as usize;
-    unsafe { dealloc(raw_buf_with_sz, Layout::from_size_align(sz, 8).unwrap()) };
+    let buf = unsafe { syms.byte_sub(mem::size_of::<u64>()).cast::<u8>().cast_mut() };
+    let size = unsafe { *buf.cast::<u64>() } as usize;
+    unsafe { dealloc(buf, Layout::from_size_align(size, 8).unwrap()) };
 }
 
 
@@ -545,6 +535,7 @@ mod tests {
             sym_type: SymType::Function,
             file_offset: Some(1337),
             module: Some(OsStr::new("/tmp/foobar.so").into()),
+            _non_exhaustive: (),
         }]];
         test(syms);
 
@@ -557,6 +548,7 @@ mod tests {
                 sym_type: SymType::Function,
                 file_offset: Some(1337),
                 module: Some(OsStr::new("/tmp/foobar.so").into()),
+                _non_exhaustive: (),
             },
             SymInfo {
                 name: "sym2".into(),
@@ -565,6 +557,7 @@ mod tests {
                 sym_type: SymType::Undefined,
                 file_offset: Some(1338),
                 module: Some(OsStr::new("other.so").into()),
+                _non_exhaustive: (),
             },
         ]];
         test(syms);
@@ -578,6 +571,7 @@ mod tests {
                 sym_type: SymType::Function,
                 file_offset: Some(1337),
                 module: Some(OsStr::new("/tmp/foobar.so").into()),
+                _non_exhaustive: (),
             }],
             vec![SymInfo {
                 name: "sym2".into(),
@@ -586,6 +580,7 @@ mod tests {
                 sym_type: SymType::Undefined,
                 file_offset: Some(1338),
                 module: Some(OsStr::new("other.so").into()),
+                _non_exhaustive: (),
             }],
         ];
         test(syms);
@@ -598,6 +593,7 @@ mod tests {
             sym_type: SymType::Function,
             file_offset: Some(1337),
             module: Some(OsStr::new("/tmp/foobar.so").into()),
+            _non_exhaustive: (),
         };
         let syms = vec![(0..200).map(|_| sym.clone()).collect()];
         test(syms);
