@@ -192,9 +192,9 @@ fn normalize(normalize: args::normalize::Normalize) -> Result<()> {
 fn print_frame(
     name: &str,
     addr_info: Option<(Addr, Addr, usize)>,
-    code_info: &Option<symbolize::CodeInfo>,
+    code_info: Option<&symbolize::CodeInfo>,
 ) {
-    let code_info = code_info.as_ref().map(|code_info| {
+    let code_info = code_info.map(|code_info| {
         let path = code_info.to_path();
         let path = path.display();
 
@@ -318,9 +318,13 @@ fn symbolize(symbolize: args::symbolize::Symbolize) -> Result<()> {
                 inlined,
                 ..
             }) => {
-                print_frame(&name, Some((input_addr, addr, offset)), &code_info);
+                print_frame(
+                    &name,
+                    Some((input_addr, addr, offset)),
+                    code_info.as_deref(),
+                );
                 for frame in inlined.iter() {
-                    print_frame(&frame.name, None, &frame.code_info);
+                    print_frame(&frame.name, None, frame.code_info.as_ref());
                 }
             }
             symbolize::Symbolized::Unknown(..) => {
