@@ -325,6 +325,7 @@ impl Symbolize for DwarfResolver {
                 lang: unit.language().into(),
                 code_info: None,
                 inlined: Box::new([]),
+                _non_exhaustive: (),
             }
         } else {
             // Fall back to checking ELF for the symbol corresponding to
@@ -508,7 +509,7 @@ impl<'dwarf> Units<'dwarf> {
             Vec::new()
         };
 
-        sym.code_info = Some(direct_code_info);
+        sym.code_info = Some(Box::new(direct_code_info));
         sym.inlined = inlined.into_boxed_slice();
 
         Ok(())
