@@ -183,6 +183,7 @@ fn find_sym<'elf>(
                         // information.
                         code_info: None,
                         inlined: Box::new([]),
+                        _non_exhaustive: (),
                     };
                     return Ok(Some(sym))
                 }
@@ -971,8 +972,9 @@ where
     /// A cache for relevant parts of the ELF file.
     // SAFETY: We must not hand out references with a 'static lifetime to
     //         this member. Rather, they should never outlive `self`.
-    //         Furthermore, this member has to be listed before `_mmap`
-    //         to make sure we never end up with a dangling reference.
+    //         Furthermore, this member has to be listed before
+    //         `_backend` to make sure we never end up with a dangling
+    //         reference.
     cache: Cache<'static, B::ImplTy<'static>>,
     /// The "module" that this parser represents.
     ///
@@ -1064,11 +1066,10 @@ impl ElfParser<Mmap> {
 
 impl ElfParser<StaticMem> {
     /// Create an `ElfParser` from a region of static memory.
-    pub(crate) fn from_mem(mem: StaticMem) -> Self {
+    pub(crate) fn from_mem(mem: StaticMem, module: OsString) -> Self {
         Self {
             cache: Cache::new(mem),
-            // TODO: Should provide the module.
-            module: None,
+            module: Some(module),
             _backend: mem,
         }
     }
