@@ -2,7 +2,6 @@ use std::backtrace::Backtrace;
 use std::backtrace::BacktraceStatus;
 use std::borrow::Borrow;
 use std::borrow::Cow;
-use std::env;
 use std::error::Error as StdError;
 use std::fmt::Debug;
 use std::fmt::Display;
@@ -15,7 +14,9 @@ use std::str;
 
 
 mod private {
-    use super::*;
+    use super::io;
+    use super::str;
+    use super::Error;
 
     pub trait Sealed {}
 
@@ -98,38 +99,6 @@ impl IntoCowStr for String {
 }
 
 
-trait MaybeCapture {
-    fn maybe_capture() -> Backtrace;
-}
-
-impl MaybeCapture for Backtrace {
-    fn maybe_capture() -> Backtrace {
-        if cfg!(debug_assertions) {
-            // For testing purposes we need to circumvent the standard
-            // library's caching logic, which otherwise could cause
-            // flakiness in our tests, as they'd be impacted by
-            // unrelated errors that check for backtrace capture.
-            let enabled = match env::var("RUST_LIB_BACKTRACE") {
-                Ok(s) => s != "0",
-                Err(_) => match env::var("RUST_BACKTRACE") {
-                    Ok(s) => s != "0",
-                    Err(_) => false,
-                },
-            };
-            if enabled {
-                Backtrace::force_capture()
-            } else {
-                Backtrace::disabled()
-            }
-        } else {
-            // Just use the standard capture infrastructure and its caching
-            // logic.
-            Self::capture()
-        }
-    }
-}
-
-
 enum ErrorImpl {
     #[cfg(feature = "dwarf")]
     Dwarf {
@@ -343,7 +312,8 @@ pub enum ErrorKind {
     /// The I/O operation's timeout expired, causing it to be canceled.
     TimedOut,
     /// An error returned when an operation could not be completed
-    /// because a call to [`write`] returned [`Ok(0)`].
+    /// because a call to [`write`][std::io::Write::write] returned
+    /// [`Ok(0)`][Ok].
     WriteZero,
     /// This operation is unsupported on this platform.
     Unsupported,
@@ -414,9 +384,9 @@ impl Display for ErrorKind {
 /// assert!(inner_err.to_string().starts_with("No such file or directory"));
 /// ```
 ///
-/// For convenient reporting, the [`Display`][std::fmt::Display]
-/// representation takes care of reporting the complete error chain when
-/// the alternate flag is set:
+/// For convenient reporting, the [`Display`] representation takes care
+/// of reporting the complete error chain when the alternate flag is
+/// set:
 /// ```
 /// # use std::fs::File;
 /// # use std::error::Error as _;
@@ -428,8 +398,8 @@ impl Display for ErrorKind {
 /// println!("{err:#}");
 /// ```
 ///
-/// The [`Debug`][std::fmt::Debug] representation similarly will print
-/// the entire error chain, but will do so in a multi-line format:
+/// The [`Debug`] representation similarly will print the entire error
+/// chain, but will do so in a multi-line format:
 /// ```
 /// # use std::fs::File;
 /// # use std::error::Error as _;
@@ -563,7 +533,7 @@ impl From<gimli::Error> for Error {
             error: Box::new(ErrorImpl::Dwarf {
                 error: other,
                 #[cfg(feature = "backtrace")]
-                backtrace: Backtrace::maybe_capture(),
+                backtrace: Backtrace::capture(),
             }),
         }
     }
@@ -575,7 +545,7 @@ impl From<io::Error> for Error {
             error: Box::new(ErrorImpl::Io {
                 error: other,
                 #[cfg(feature = "backtrace")]
-                backtrace: Backtrace::maybe_capture(),
+                backtrace: Backtrace::capture(),
             }),
         }
     }
@@ -591,7 +561,7 @@ impl From<Box<dyn StdError + Send + Sync + 'static>> for Error {
                 // because no trait exposes it. So the best we can do is capture
                 // a backtrace ourselves. Sigh.
                 #[cfg(feature = "backtrace")]
-                backtrace: Backtrace::maybe_capture(),
+                backtrace: Backtrace::capture(),
             }),
         }
     }
@@ -762,7 +732,7 @@ mod tests {
     use std::env;
     use std::mem::size_of;
 
-    use test_log::test as forked_test;
+    use test_fork::fork;
     use test_log::test;
     use test_tag::tag;
 
@@ -899,14 +869,15 @@ Caused by:
     /// that the file name is contained in the backtrace. For that reason we
     /// only run it on debug builds (represented by the `debug_assertions`
     /// proxy cfg).
-    #[forked_test]
+    #[fork]
+    #[test]
     fn error_backtrace() {
         if !cfg!(debug_assertions) {
             return
         }
 
         // Ensure that we capture a backtrace.
-        let () = env::set_var("RUST_LIB_BACKTRACE", "1");
+        let () = unsafe { env::set_var("RUST_LIB_BACKTRACE", "1") };
 
         let err = io::Error::new(io::ErrorKind::InvalidData, "some invalid data");
         let err = Error::from(err);
@@ -919,10 +890,11 @@ Caused by:
 
     /// Make sure that we do not emit backtraces in errors when
     /// the `RUST_LIB_BACKTRACE` environment variable is not present.
-    #[forked_test]
+    #[fork]
+    #[test]
     fn error_no_backtrace1() {
-        let () = env::remove_var("RUST_BACKTRACE");
-        let () = env::remove_var("RUST_LIB_BACKTRACE");
+        let () = unsafe { env::remove_var("RUST_BACKTRACE") };
+        let () = unsafe { env::remove_var("RUST_LIB_BACKTRACE") };
 
         let err = io::Error::new(io::ErrorKind::InvalidData, "some invalid data");
         let err = Error::from(err);
@@ -933,10 +905,11 @@ Caused by:
 
     /// Make sure that we do not emit backtraces in errors when
     /// the `RUST_LIB_BACKTRACE` environment variable is "0".
-    #[forked_test]
+    #[fork]
+    #[test]
     fn error_no_backtrace2() {
-        let () = env::remove_var("RUST_BACKTRACE");
-        let () = env::set_var("RUST_LIB_BACKTRACE", "0");
+        let () = unsafe { env::remove_var("RUST_BACKTRACE") };
+        let () = unsafe { env::set_var("RUST_LIB_BACKTRACE", "0") };
 
         let err = io::Error::new(io::ErrorKind::InvalidData, "some invalid data");
         let err = Error::from(err);
