@@ -206,6 +206,7 @@ impl Symbolize for GsymResolver<'_> {
                 lang,
                 code_info: None,
                 inlined: Box::new([]),
+                _non_exhaustive: (),
             };
             let () = self.fill_code_info(&mut sym, addr, opts, sym_addr, &info)?;
 
@@ -315,7 +316,7 @@ impl GsymResolver<'_> {
             }
         }
 
-        sym.code_info = Some(line_tab_info);
+        sym.code_info = Some(Box::new(line_tab_info));
         sym.inlined = inlined.into_boxed_slice();
 
         Ok(())
