@@ -113,13 +113,15 @@ pub use symbolizer::ProcessDispatch;
 pub use symbolizer::ProcessMemberInfo;
 pub use symbolizer::Symbolizer;
 
+pub(crate) use symbolizer::symbolize_with_resolver;
+pub(crate) use symbolizer::Resolver;
+
 // Strictly speaking these types are applicable to the entire crate, but right
 // now they are only used as part of the symbolization APIs, so we re-export
 // them through this module only.
 pub use crate::maps::EntryPath as ProcessMemberPath;
 pub use crate::maps::PathName as ProcessMemberType;
 
-use crate::normalize;
 use crate::Addr;
 use crate::Result;
 
@@ -268,12 +270,20 @@ impl CodeInfo<'_> {
 
     /// Convert this object into one with all references converted into
     /// guaranteed owned (i.e., heap allocated) members.
-    pub fn to_owned(&self) -> CodeInfo<'static> {
+    pub fn into_owned(self) -> CodeInfo<'static> {
+        let Self {
+            dir,
+            file,
+            line,
+            column,
+            _non_exhaustive: (),
+        } = self;
+
         CodeInfo {
-            dir: self.dir.as_ref().map(|dir| Cow::Owned(dir.to_path_buf())),
-            file: Cow::Owned(self.file.to_os_string()),
-            line: self.line,
-            column: self.column,
+            dir: dir.map(|dir| Cow::Owned(dir.into_owned())),
+            file: Cow::Owned(file.into_owned()),
+            line,
+            column,
             _non_exhaustive: (),
         }
     }
@@ -292,6 +302,24 @@ pub struct InlinedFn<'src> {
     pub _non_exhaustive: (),
 }
 
+impl InlinedFn<'_> {
+    /// Convert this object into one with all references converted into
+    /// guaranteed owned (i.e., heap allocated) members.
+    pub fn into_owned(self) -> InlinedFn<'static> {
+        let Self {
+            name,
+            code_info,
+            _non_exhaustive: (),
+        } = self;
+
+        InlinedFn {
+            name: Cow::Owned(name.into_owned()),
+            code_info: code_info.map(CodeInfo::into_owned),
+            _non_exhaustive: (),
+        }
+    }
+}
+
 
 /// The source code language from which a symbol originates.
 #[derive(Clone, Copy, Default, Debug, PartialEq)]
@@ -321,9 +349,15 @@ pub struct ResolvedSym<'src> {
     /// The source code language from which the symbol originates.
     pub lang: SrcLang,
     /// Source code location information.
-    pub code_info: Option<CodeInfo<'src>>,
+    pub code_info: Option<Box<CodeInfo<'src>>>,
     /// Inlined function information.
     pub inlined: Box<[InlinedFn<'src>]>,
+    /// The struct is non-exhaustive and open to extension.
+    // TODO: In the future we may want to make this type exhaustive to
+    //       allow users to construct it easily themselves, in order to
+    //       enable usage of custom "resolvers".
+    #[doc(hidden)]
+    pub _non_exhaustive: (),
 }
 
 
@@ -336,10 +370,11 @@ pub struct Sym<'src> {
     ///
     /// Typically this would be the path to a executable or shared
     /// object. Depending on the symbol source this member may not be
-    /// present or it could also just be a file name without path. In
-    /// case of an ELF file contained inside an APK, this will be an
-    /// Android style path of the form `<apk>!<elf-in-apk>`. E.g.,
-    /// `/root/test.apk!/lib/libc.so`.
+    /// present or it could also just be a file name without path or a
+    /// symbolic name such as `[vdso]` representing the vDSO or `[bpf]`
+    /// for symbols in BPF programs. In case of an ELF file contained
+    /// inside an APK, this will be an Android style path of the form
+    /// `<apk>!<elf-in-apk>`. E.g., `/root/test.apk!/lib/libc.so`.
     pub module: Option<Cow<'src, OsStr>>,
     /// The address at which the symbol is located (i.e., its "start").
     ///
@@ -360,7 +395,7 @@ pub struct Sym<'src> {
     /// The symbol's size, if available.
     pub size: Option<usize>,
     /// Source code location information for the symbol.
-    pub code_info: Option<CodeInfo<'src>>,
+    pub code_info: Option<Box<CodeInfo<'src>>>,
     /// Inlined function information, if requested and available.
     ///
     /// Availability depends on both the underlying symbolization source (e.g.,
@@ -378,6 +413,37 @@ pub struct Sym<'src> {
     pub _non_exhaustive: (),
 }
 
+impl Sym<'_> {
+    /// Convert this object into one with all references converted into
+    /// guaranteed owned (i.e., heap allocated) members.
+    pub fn into_owned(self) -> Sym<'static> {
+        let Self {
+            name,
+            module,
+            addr,
+            offset,
+            size,
+            code_info,
+            inlined,
+            _non_exhaustive,
+        } = self;
+
+        Sym {
+            name: Cow::Owned(name.into_owned()),
+            module: module.map(|module| Cow::Owned(module.into_owned())),
+            addr,
+            offset,
+            size,
+            code_info: code_info.map(|info| Box::new(info.into_owned())),
+            inlined: Vec::from(inlined)
+                .into_iter()
+                .map(InlinedFn::into_owned)
+                .collect::<Box<[_]>>(),
+            _non_exhaustive: (),
+        }
+    }
+}
+
 
 /// The reason why symbolization failed.
 ///
@@ -431,17 +497,6 @@ impl Display for Reason {
     }
 }
 
-impl From<normalize::Reason> for Reason {
-    #[inline]
-    fn from(reason: normalize::Reason) -> Self {
-        match reason {
-            normalize::Reason::Unmapped => Self::Unmapped,
-            normalize::Reason::MissingComponent => Self::MissingComponent,
-            normalize::Reason::Unsupported => Self::Unsupported,
-        }
-    }
-}
-
 
 /// An enumeration used as reporting vehicle for address symbolization.
 // We keep this enum as exhaustive because additions to it, should they occur,
@@ -481,8 +536,9 @@ impl<'src> Symbolized<'src> {
 
 
 /// A trait helping with upcasting into a `dyn Symbolize`.
-// TODO: This trait is currently necessary because Rust does not yet support
-//       trait upcasting on stable (check `trait_upcasting` feature).
+// TODO: This trait is currently necessary because our MSRV is <1.86.
+//       Once we upgrade to that, we can make use of "native" upcasting
+//       support on stable.
 #[doc(hidden)]
 pub trait AsSymbolize {
     fn as_symbolize(&self) -> &dyn Symbolize;
@@ -582,6 +638,40 @@ mod tests {
         );
     }
 
+    /// Test forcing a double check of all `Sym` size changes.
+    #[cfg(target_pointer_width = "64")]
+    #[test]
+    fn sym_size() {
+        assert_eq!(size_of::<Sym>(), 104);
+    }
+
+    /// Check that [`Sym::into_owned`] works as expected.
+    #[test]
+    fn owned_conversion() {
+        let sym = Sym {
+            name: Cow::Borrowed("test"),
+            module: Some(Cow::Borrowed(OsStr::new("module"))),
+            addr: 1337,
+            offset: 42,
+            size: None,
+            code_info: None,
+            inlined: Box::new([InlinedFn {
+                name: Cow::Borrowed("inlined_test"),
+                code_info: Some(CodeInfo {
+                    dir: Some(Cow::Borrowed(Path::new("/tmp/some-dir"))),
+                    file: Cow::Borrowed(OsStr::new("test.c")),
+                    line: Some(1337),
+                    column: None,
+                    _non_exhaustive: (),
+                }),
+                _non_exhaustive: (),
+            }]),
+            _non_exhaustive: (),
+        };
+
+        assert_eq!(sym, sym.clone().into_owned());
+    }
+
     /// Check that the [`Input::map`] helper works as expected.
     #[test]
     fn input_mapping() {
@@ -600,21 +690,6 @@ mod tests {
         }
     }
 
-    /// Check that we can convert `normalize::Reason` objects into
-    /// `symbolize::Reason` objects.
-    #[test]
-    fn reason_conversion() {
-        assert_eq!(Reason::from(normalize::Reason::Unmapped), Reason::Unmapped);
-        assert_eq!(
-            Reason::from(normalize::Reason::MissingComponent),
-            Reason::MissingComponent
-        );
-        assert_eq!(
-            Reason::from(normalize::Reason::Unsupported),
-            Reason::Unsupported
-        );
-    }
-
     /// Test the `Symbolized::*_sym()` conversion methods for the `Unknown`
     /// variant.
     #[test]
